#include "vga.h"
#include "system.h"
#include "io.h"
#include <stdio.h>

void Vga::enter_read_mode( )
{
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_RAM_WRITE, 0x00  );
}

void Vga::locate_cursor( const int& x, const int& y )
{
    unsigned int command = 0;
    
    // build command
    command = ( x ) + ( y << 10 );
    
    // send the command to the VGA memory-mapped I/O address
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_MOVE_CURSOR, command );
}

void Vga::get_cursor_position( int& pos_x, int& pos_y )
{
    pos_x = m_cursor_pos_x;
    pos_y = m_cursor_pos_y;
}

void Vga::move_cursor( const int& dir_x, const int& dir_y )
{
    // apply location
    m_cursor_pos_x += dir_x;
    m_cursor_pos_y += dir_y;
    
    // check whether the direction exceeds the corner location
    if( m_cursor_pos_x < 0 )
    {
        m_cursor_pos_x = 0;
    }
    else if( m_cursor_pos_x > m_width )
    {
        m_cursor_pos_x = m_width;
    }
    
    if( m_cursor_pos_y < 0 )
    {
        m_cursor_pos_y = 0;
    }
    else if( m_cursor_pos_y > m_height )
    {
        m_cursor_pos_y = m_height;
    }
    
    // send the command
    locate_cursor( m_cursor_pos_x, m_cursor_pos_y );
}

void Vga::set_resolution( const int& width, const int& height )
{
    m_width = width;
    m_height = height;
}

void Vga::store_image( const int& sprite_id, const unsigned short * buf, const int& len )
{
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_RAM_WRITE, 0xFFFFFFFF );
    
    for( int i = 0; i < len * 4; i += 4 )
    {
        IOWR_32DIRECT( SRAM_BASE, ADDRESS_IMAGE_SPRITE_0 + sprite_id * IMAGE_SIZE + i, *( buf + i / 4 ) );
    }
    
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_RAM_WRITE, 0x00 );
}

void Vga::hide_image( const int& sprite_id )
{
    //int data = 0;
    //data = IORD_32DIRECT( VGA_BASE, ADDRESS_OFFSET_NORMAL_IMAGE_SET  );
    m_data |= 1 << ( 16 + sprite_id  );
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_NORMAL_IMAGE_SET, m_data );
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * sprite_id, 0x00 );
}

void Vga::hide_image_all( )
{
    m_data = 0xFFFF0000;
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_NORMAL_IMAGE_SET, m_data );
    for( int i = 0; i < 16; i++ )
    {
        IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * i, 0x00 );
    }
}

void Vga::show_back_image_all(  )
{
    m_data = 0x00;
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_NORMAL_IMAGE_SET, m_data );
    for( int i = 0; i < 16; i++ )
    {
        IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * i, 0x00 );
    }
}

void Vga::show_front_image_all( )
{
    m_data = 0xFFFF;
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_NORMAL_IMAGE_SET, m_data );
    for( int i = 0; i < 16; i++ )
    {
        IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * i, 0x00 );
    }
}

void Vga::show_back_image( const int& sprite_id )
{
    //int data = 0;
    m_data = IORD_32DIRECT( VGA_BASE, ADDRESS_OFFSET_NORMAL_IMAGE_SET );
    m_data = ~m_data;
    m_data |= 1 << ( 16 + sprite_id  );
    m_data |= 1 << sprite_id;
    m_data = ~m_data;
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_NORMAL_IMAGE_SET, m_data );
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * sprite_id, 0x00 );
}

void Vga::show_front_image( const int& sprite_id )
{
    //int data = 0;
    //data = IORD_32DIRECT( VGA_BASE, ADDRESS_OFFSET_NORMAL_IMAGE_SET );
    //printf( "data after read: %02X\n", data );
    m_data |= 1 << sprite_id;
    //printf( "data after revert: %02X\n", data );
    m_data = ~m_data;
    //printf( "data after bit operation over 16 bits: %02X\n", data );
    m_data |= 1 << ( 16 + sprite_id  );
    //printf( "data after bit operation: %02X\n", data );
    m_data = ~m_data;
    //printf( "data after bit operation: %02X\n", data );
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_NORMAL_IMAGE_SET, m_data );
    //printf( "data to be written: %02X\n", data );
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * sprite_id, 0x00 );
}

void Vga::turn_back_image( const int& sprite_id )
{
    //hide_image( sprite_id );
    show_back_image( sprite_id );
    
    //IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * sprite_id, 0x00 );
    delay( 70000 );
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * sprite_id, 0x01 );
    delay( 7000 );
}

void Vga::turn_front_image( const int& sprite_id )
{
    //hide_image( sprite_id );
    show_front_image( sprite_id );
    
    //IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * sprite_id, 0x00 );    
    delay( 70000 );
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * sprite_id, 0x02 );
    delay( 7000 );
}

void Vga::blink_back_image( const int& sprite_id )
{
    for( int i = 0; i < 1; i++ )
    {
        show_back_image( sprite_id );
        delay( 700000 );
        hide_image( sprite_id );
        delay( 700000 );
    }
}

void Vga::blink_front_image( const int& sprite_id )
{
    for( int i = 0; i < 3; i++ )
    {
        show_front_image( sprite_id );
        delay( 700000 );
        hide_image( sprite_id );
        delay( 700000 );
    }
}

void Vga::fade_out_image( const int& sprite_id )
{
    show_front_image( sprite_id );
    delay( 100000 );
    IOWR_32DIRECT( VGA_BASE, ADDRESS_OFFSET_EFFECT_SPRITE_0 + 4 * sprite_id, 0x03 );
    //delay( 10000000 );
}
