#ifndef VGA_H_
#define VGA_H_

class Vga
{
    public:
    ~Vga( ){ }
    
    static Vga& create( ){ static Vga m_vga;  return m_vga; }
    
    void enter_read_mode( );
    void locate_cursor( const int& x, const int& y );
    void get_cursor_position( int& pos_x, int& pos_y );
    void move_cursor( const int& dir_x, const int& dir_y );
    void store_image( const int& sprite_id, const unsigned short * buf, const int& len );
    void hide_image( const int& sprite_id );
    void hide_image_all( );
    void show_back_image_all(  );
    void show_front_image_all( );
    void show_back_image( const int& sprite_id );
    void show_front_image( const int& sprite_id );
    void turn_back_image( const int& sprite_id );
    void turn_front_image( const int& sprite_id );
    void blink_back_image( const int& sprite_id );
    void blink_front_image( const int& sprite_id );
    void fade_out_image( const int& sprite_id );
    
    void set_resolution( const int& width, const int& height );

    private:  
    Vga( ){ m_data = 0; }
    
    static const unsigned int N_SPRITE = 16;
    static const unsigned int IMAGE_SIZE = 0x8000;
    
    static const unsigned int ADDRESS_OFFSET_MOVE_CURSOR         = 0x00;
    static const unsigned int ADDRESS_OFFSET_RAM_WRITE           = 0x08;
    static const unsigned int ADDRESS_OFFSET_NORMAL_IMAGE_SET    = 0x0C;
//    static const unsigned int ADDRESS_OFFSET_EFFECT_SPRITE[ N_SPRITE ] =
//    {
//        0x10,
//        0x14,
//        0x18,
//        0x1C,
//        0x20,
//        0x24,
//        0x28,
//        0x2C,
//        0x30,
//        0x34,
//        0x38,
//        0x3C,
//        0x40,
//        0x44,
//        0x48,
//        0x4C
//    };
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_0     = 0x10;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_1     = 0x14;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_2     = 0x18;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_3     = 0x1C;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_4     = 0x20;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_5     = 0x24;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_6     = 0x28;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_7     = 0x2C;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_8     = 0x30;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_9     = 0x34;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_10    = 0x38;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_11    = 0x3C;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_12    = 0x40;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_13    = 0x44;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_14    = 0x48;
    static const int ADDRESS_OFFSET_EFFECT_SPRITE_15    = 0x4C;

//    static unsigned int ADDRESS_IMAGE_SPRITE[ N_SPRITE ] =
//    {
//        0x00000000,
//        0x00008000,
//        0x00010000,
//        0x00018000,
//        0x00020000,
//        0x00028000,
//        0x00030000,
//        0x00038000,
//        0x00040000,
//        0x00048000,
//        0x00050000,
//        0x00058000,
//        0x00060000,
//        0x00068000,
//        0x00070000,
//        0x00078000
//    };
    
    static const unsigned int ADDRESS_IMAGE_SPRITE_0    = 0x00000000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_1    = 0x00008000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_2    = 0x00010000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_3    = 0x00018000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_4    = 0x00020000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_5    = 0x00028000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_6    = 0x00030000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_7    = 0x00038000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_8    = 0x00040000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_9    = 0x00048000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_10   = 0x00050000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_11   = 0x00058000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_12   = 0x00060000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_13   = 0x00068000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_14   = 0x00070000;
    static const unsigned int ADDRESS_IMAGE_SPRITE_15   = 0x00078000;

    int m_width;
    int m_height;
    int m_cursor_pos_x;
    int m_cursor_pos_y;
    int m_data;

    void delay( int time ){ volatile int delay = time; while( delay-- ); }

};

#endif /*VGA_H_*/
